
# q-expansion of newform 2574.2.a.l, downloaded from the LMFDB on 11 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2574
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1], [0], [2], [-2], [-1], [-1], [-4], [2], [4], [4], [-6], [-2], [-2], [-4], [8], [-4], [-4], [-14], [2], [0], [-10], [-16], [12], [-10], [14], [12], [-4], [-4], [-14], [4], [4], [-12], [10], [0], [-22], [10], [2], [2], [-8], [-20], [12], [-2], [12], [-6], [14], [-16], [-12], [26], [12], [-6], [-16], [-24], [-10], [0], [-12], [0], [-24], [-2], [-22], [-18], [24], [2], [-6], [20], [-6], [-22], [-14], [-2], [0], [22], [18], [16], [-24], [-10], [-2], [8], [12], [-18], [-18], [30], [20], [26], [8], [30], [24], [4], [6], [14], [22], [-6], [-8], [0], [34], [24], [-38], [0], [18], [-4], [-32], [18], [8], [18], [-36], [16], [-32], [34], [-12], [-6], [-24], [26], [20], [22], [-6], [-10], [6], [24], [2], [24], [8], [40], [46], [-2], [-24], [44], [-30], [-16], [-14], [-20], [16], [50], [10], [0], [-12], [-18], [14], [18], [-42], [-6], [44], [-24], [42], [-6], [-20], [20], [14], [8], [22], [-56], [-32], [0], [-10], [36], [4], [-12], [28], [-20], [40], [-6], [-42], [-30], [36], [0], [22], [-16], [-42], [56], [-40], [-42], [-34], [8], [36], [-58], [60], [-18], [4], [-16], [-22], [30], [40], [14], [-62], [60], [14], [-14], [64], [-20], [-18], [-2], [6], [48], [2], [-36], [-20], [-18], [12], [-12], [34], [34], [-30], [-36], [-10], [-60], [-14], [26], [-36], [12], [26], [68], [-6], [-60], [54], [12], [-4], [-36], [-56], [26], [-40], [-20], [-8], [-34], [-70], [-2], [30], [-14], [-48], [54], [-48], [24], [44], [36], [-58], [0], [2], [-20], [-52], [72], [-66], [58], [-20], [-28], [-36], [32], [20], [6], [2], [-20], [-70], [36], [-14], [-28], [66], [-2], [40], [70], [28], [44], [26], [-62], [12], [-62], [44], [-8], [-46], [-2], [78], [-48], [18], [-6], [-62], [-52], [-46], [2], [46], [-40], [-2], [70], [36], [78], [-30], [56], [44], [-70], [-68], [46], [-22], [-48], [-6], [2], [22], [-48], [52], [-8], [10], [68], [-38], [32], [-24], [44], [-48], [-22], [-54], [-42], [32], [64], [76], [54], [-44], [50], [-40], [-10], [8], [-38], [-24], [0], [-40], [-26], [20], [48], [70], [-36], [44], [2], [72], [46], [-86], [-38], [10], [58], [-24], [-68], [-46], [72], [-8], [12], [2], [-28], [-34], [14], [-26], [-32], [70], [-28], [10], [-4], [26], [-28], [-6], [-70], [0], [4], [6], [-82], [-42], [4], [18], [-40], [-88], [-84], [24], [-72], [46], [36], [-72], [-76], [-8], [22], [86], [-54], [-26], [84], [-64], [-32], [72], [-16], [34], [-20], [-56], [34], [-52], [42], [6], [-54], [78], [66], [30], [-56], [78], [-58], [50], [40], [-50], [6], [-36], [56], [-48], [-86], [-38], [-96], [40], [-74], [-58], [48], [62], [-34], [-10], [40], [-30], [-70], [70], [-12], [50], [64], [24], [-60], [-42], [-12], [-104], [12], [-10], [-52], [-64], [-90], [48], [-12], [-30], [-42], [96], [-22], [-62], [-36]]
